function  W=TPS_weights(xv,yv,X,Y)

%--------------------------------------------------------------------------
% OBJECT.
%--------------------------------------------------------------------------
% Numerical cubature of a polygon (with vertices (xv, yv), of the integrand
% "f", using the nodes (X,Y) by means of TPS splines.
%--------------------------------------------------------------------------
% INPUT.
%--------------------------------------------------------------------------
% xv, yv: column vectors of abscissas and ordinates of the vertices of the
%         polygon, ordered counterclockwise; no need of last vertex to be
%         repeated (as in inpolygon)
% X, Y  : cubature nodes
%--------------------------------------------------------------------------
% OUTPUT.
%--------------------------------------------------------------------------
% W: cubature weights (relative to (X,Y)).
%--------------------------------------------------------------------------
% RELATED WORK.
%--------------------------------------------------------------------------
% Alvise Sommariva, Marco Vianello
% "Meshless cubature by Green?s formula"
% Applied Mathematics and Computation 183 (2006) 1098?1107.
%--------------------------------------------------------------------------
% Copyrights.
%--------------------------------------------------------------------------
%% Copyright (C) 2006-2019 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: May 2006 - September 24, 2019.
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------
% TROUBLESHOOTING.
%--------------------------------------------------------------------------

warning off;

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% MAIN PROGRAM STARTS HERE
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%--------------------------------------------------------------------------
% INTERPOLATION.
%--------------------------------------------------------------------------

dim=2; % dimension of the domain (it is in R^2)
Npts=length(X);
tps=@(r) r.*r.*log(r+10^(-20));
points=[X Y];
rbf_cmp=tps(points2distances(points));
A=[rbf_cmp  ones(Npts,1)  points; ...
    ones(1,Npts) zeros(1,1) zeros(1,dim) ; ...
    points' zeros(dim,1) zeros(dim,dim)];

moms=TPS_moments(xv,yv,X,Y);

w_ext=A\moms;

%--------------------------------------------------------------------------
% INTEGRATION.
%--------------------------------------------------------------------------

W=w_ext(1:Npts);




%==========================================================================
% ATTACHED FUNCTIONS.
%==========================================================================

function distances = points2distances(points)

% Create distance matrix from point matrix.

% Get dimensions.
[numpoints,dim]=size(points);

% All inner products between points.
distances=points*points';

% Vector of squares of norms of points.
lsq=diag(distances);

% Distance matrix.
distances=sqrt(repmat(lsq,1,numpoints)+repmat(lsq,1,numpoints)'-2*distances);









function [INTEGRATION_RESULT]=green_integration(COEFFS,BOUNDARY_PTS,PTS)

%==========================================================================
% COEFFS: RADIAL BASIS FUNCTION COEFFICIENTS; SIZE Lx1.
% BOUNDARY POINTS: POINTS THAT DEFINE THE POLYGONAL DOMAIN (IN SEQUENCE!);
%         SIZE Mx2.
% PTS: RADIAL BASIS FUNCTION CENTERS; SIZE Nx2.
%==========================================================================

L=size(COEFFS,1);
N=size(PTS,1);

%==========================================================================
% EVALUATION INTEGRAL RELATIVE TO RBF PART.
%==========================================================================
RBF_INTEGRAL_PTS=integral_RBF(PTS,BOUNDARY_PTS);
PTS_COEFFS=COEFFS(1:length(PTS(:,1)),1);
INTEGRATION_RESULT_RBF=PTS_COEFFS'*RBF_INTEGRAL_PTS;

%==========================================================================
% EVALUATION INTEGRAL RELATIVE TO POLYNOMIAL PART.
%==========================================================================

if norm(BOUNDARY_PTS(1,:)-BOUNDARY_PTS(end,:)) > 0
    BOUNDARY_PTS=[BOUNDARY_PTS; BOUNDARY_PTS(1,:)];
end

% intfcn,N,polygon_sides,rotation,P,Q,quadf,cubature_type,varargin
[xyw,xvc,yvc,pgon,tri]=polygauss_2018(1,BOUNDARY_PTS(:,1),...
    BOUNDARY_PTS(:,2));
XX=xyw(:,1); YY=xyw(:,2); WW=xyw(:,3);
LOC_POLYNOMIAL_INTEGRAL(1,1)=sum(WW);
LOC_POLYNOMIAL_INTEGRAL(2,1)=XX'*WW;
LOC_POLYNOMIAL_INTEGRAL(3,1)=YY'*WW;

POLY_COEFFS=COEFFS(end-2:end,1);
INTEGRATION_RESULT_POLYNOMIALS=POLY_COEFFS'*LOC_POLYNOMIAL_INTEGRAL;

%==========================================================================
% EVALUATION WHOLE INTEGRAL.
%==========================================================================

INTEGRATION_RESULT=INTEGRATION_RESULT_RBF+INTEGRATION_RESULT_POLYNOMIALS;

%==========================================================================
% RBF AND POLYNOMIALS MOMENTS VECTOR.
%==========================================================================

moms=[RBF_INTEGRAL_PTS; LOC_POLYNOMIAL_INTEGRAL];



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%  FUNCTIONS NEEDED BY "green_integration".
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%=======================================================================
% FUNCTION CALLED DIRECTLY BY "green_integration".
%=======================================================================

function TOTAL_INTEGRAL=integral_RBF(LOC_PTS,BOUNDARY_PTS)
%=======================================================================
% IT COMPUTES THE INTEGRAL OF A RADIAL BASIS WITH CENTER IN "LOC_PTS",
% FOR A DOMAIN DEFINED BY THE SEQUENCE (!) "BOUNDARY_PTS".
%=======================================================================
%=======================================================================
% THE VECTOR "BOUNDARY_PTS" IS PERIODIC FOR COMPUTATIONAL SEMPLICITY
% IN CLOSING THE POLYGON. REMEMBER THAT "BOUNDARY_PTS" HAS SIZE "Mx2".
%=======================================================================

M=size(BOUNDARY_PTS,1);

RBF_TYPE=5;         % TPS SETTINGS.
RBF_ORDER=2;
RBF_SCALE=1;

BOUNDARY_PTS=[BOUNDARY_PTS; BOUNDARY_PTS(1,:)];

TOTAL_INTEGRAL=zeros(size(LOC_PTS(:,1)));

for INDEX=1:M
    
    % POINTS DEFINING THE (LOCAL) POLYGONAL SIDE.
    A=BOUNDARY_PTS(INDEX,:);
    B=BOUNDARY_PTS(INDEX+1,:);
    
    % POINTS DEFINING PARAMETRIC EQUATIONS OF (LOCAL) POLYGONAL SIDE.
    
    % X(T)=P*T+Q; $T \IN [0,1]$
    P=B(1)-A(1);
    Q=A(1);
    
    % Y(T)=U*T+V    $T \IN [0,1]$
    U=B(2)-A(2);
    V=A(2);
    
    % FOR SEGMENTS THE JACOBIAN CORRESPONDS TO "U", IF THE INTEGRAL IS IN
    % "dy".
    LOC_JACOBIAN=U;
    
    % INTEGRATION OF THE PRIMITIVE (WITHOUT JACOBIAN, SEE FORMULAE FOR
    % SEGMENTS).
    
    if (LOC_JACOBIAN ~= 0)
        LOC_RBF_INTEGRAL=primitives_integrated(P,Q,U,V,LOC_PTS);
        
        % FIXING POSSIBLE TROUBLES OF NODES ON POLYGONAL SIDES.
        SS=find(isnan(LOC_RBF_INTEGRAL) == 1);
        if (isempty(SS) == 0)
            LOC_RBF_INTEGRAL(SS)=0;
        end
        
        TOTAL_INTEGRAL=TOTAL_INTEGRAL+LOC_JACOBIAN*LOC_RBF_INTEGRAL;
    end
    
end






%=======================================================================
% "primitives_integrated": CALLED BY "integral_RBF".
%=======================================================================

function INTEGRAL_RES=primitives_integrated(P,Q,U,V,LOC_PTS)

% HERE THE VARIABLES ARE x, y. THEY ARE NOT THE CARTESIAN COORDINATES BUT
% RESPECTIVELY THE RESULT OF THE EVALUATION OF THE FUNCTION AND PARAMETRIC
% VARIABLE OF THE SEGMENT "T".


% FROM PARAMETRIC COORDINATES TO CARTESIAN COORDINATES.
A=LOC_PTS(:,1)-Q;
B=-P;

C=LOC_PTS(:,2)-V;
D=-U;

A2=A.^2;
A3=A.^3;

B2=B^2;
B3=B^3;

C2=C.^2;

D2=D^2;

if (B ~= 0)
    I(:,1)=(1/9) * ( (A+B).^4 - A.^4 ) / (4*B);
else
    I(:,1)=(1/9)*A3;
end

I(:,2)=(2/3) * ( A.*C2 + (1/2)*C.*(B*C+2*A*D) + ...
    (1/3)*D*(2*B*C+A*D) + (1/4)*B*(D2) );

a1=C;
b1=D;
c1=A;
d1=B;

S=primitive_rat_arctan(a1,b1,c1,d1);
I(:,3)=(-2/3)*S;

d1=(-1/3)*A3-A.*C2;
g1=-A2*B-2*A.*C*D-B*C2;
h1=-A*B2-A*D2-2*B*C*D;
m1=(-1/3)*B3-B*D2;

a1=A2+C2;
b1=2*(A*B+C*D);
c1=B2+D2;

I(:,4)=primitive_logquad(d1,g1,h1,m1,a1,b1,c1,1)-...
    primitive_logquad(d1,g1,h1,m1,a1,b1,c1,0);

INTEGRAL_RES=sum(I,2);






%=======================================================================
% "primitive_rat_arctan": CALLED BY "primitives_integrated".
%=======================================================================

function res=primitive_rat_arctan(a,b,c,d)

%========================================================
% IT COMPUTES:
% INTEGRAL( ((a+bx)^3) * atan((c+dx)/(a+bx)); 0, 1 )
%========================================================

if b == 0
    fprintf('\n \t [WARNING]: ATAN PROBLEMS, USED DEFAULT.')
    res=zeros(size(a));
else
    a_mod=c-(a*d/b); b_mod=d/b;
    res1=int_arctan(a+b,a_mod,b_mod);
    res0=int_arctan(a,a_mod,b_mod);
    res=(1/b)*(res1-res0);
end







%=======================================================================
% "int_arctan" CALLED BY "primitive_rat_arctan".
% "primitive_rat_arctan" CALLED BY "primitives_integrated".
%=======================================================================
function res=int_arctan(x,a,b)

b2=b^2;
s=(1+b2);
x2=x.^2;
a2=a.^2;
a3=a.^3;

term1=1/(12*(s^4));

x_mod=x+(1-abs(sign(x)))*eps;
term2=3*(s^4)*(x.^4).*atan(b+(a./x_mod));

term3=a;

term4=s*x;

term5=a2*(-3+9*b2)-3*s*b*a.*x+(s^2)*x2;

term6=3*(1-6*b2+b^4)*a3.*atan((a*b+x+b2*x)./a);

term7=6*b*(-1+b2)*a3.*log(a2+2*b*a.*x+(1+b2)*x2);

res=term1*(term2+term3.*(term4.*term5+term6-term7));








%=======================================================================
% "primitive_logquad": CALLED BY "primitives_integrated".
%=======================================================================

function y=primitive_logquad(d,g,h,m,a,b,c,x)

% IT COMPUTES, ACCORDING TO MATHEMATICA, THE INDEFINITE INTEGRAL OF
%
% (d+g*x+h*(x^2)+m*(x^3))*log( sqrt(a+b*x+c*(x^2)) ).
%

b2=b.^2;
b3=b.^3;
b4=b.^4;

c2=c.^2;
c3=c.^3;
c4=c.^4;

x2=x.^2;
x3=x.^3;
x4=x.^4;

discr=sqrt(-b2+4*a.*c);

log_quad_arg=a+x*(b+c*x);

log_quad_arg=log_quad_arg+( 1-abs( sign( log_quad_arg ) ) )*eps;
log_quad=log(log_quad_arg);


term1_num=(-6)*...
    ( 24*c3.*d - 2*c2.*(3*b.*g+4*a.*h) -3*b3*m + b.*c.*(4*b.*h+9*a*m)).*x;
term(:,1)=term1_num./c3;

term2_num=(-3)*( 12*c2.*g - 4*b.*c.*h + 3*b2*m -6*a.*c*m )*x2;
term(:,2)=term2_num./c2;

term3_num=(-2)*( 8*c.*h - 3*b*m )*x3;
term(:,3)=term3_num./c;

term(:,4)=(-9)*m*x4*ones(size(a));

term5_num_1=6*discr;
term5_num_2=(-12)*...
    c3.*d + c2.*(6*b.*g+4*a.*h) + 3*b3*m -2 *b.*c.*( 2*b.*h+3*a*m );


atan_num=b+2*c*x;
atan_den_mod=discr+( 1-abs( sign( discr ) ) )*eps;
term5_num_3=abs(sign(discr)).* atan( atan_num./atan_den_mod );


term5_den=(-1)*c4;

term(:,5)=(term5_num_1.*term5_num_2.*term5_num_3)./term5_den;

term6_num_1a=3*( 4*b3.*c.*h + 12*b.*c2.*(c.*d-a.*h) - 3*b4*m - ...
    6*b2.*c.*(c.*g -2*a*m) - 6*a.*c2.*(-2*c.*g+a*m) );
term6_num_1b=log_quad;

term6_num=term6_num_1a.*term6_num_1b;
term6_den=c4;

term(:,6)=term6_num./term6_den;

term(:,7)=6*x*( 12*d+x*(6*g+x*(4*h+3*m*x)) ).*log_quad;

y=(1/72)*sum(term,2);

% THIS PART IS DUE TO THE SQUARE ROOT OF THE ARGUMENT OF THE LOG!
y=(1/2)*y;



