
function [cubature_value,X,Y]=...
    demo_2019_weights(polygon_type,function_type,cardinality_nodes)

%--------------------------------------------------------------------------
% SETTINGS.
%--------------------------------------------------------------------------

if nargin < 1
    polygon_type=4; % see attached routine "define_polygon"
end

if nargin < 2
    function_type=2; % define function, see attached routine "integrands"
end

if nargin < 3
    cardinality_nodes=1000; % number of cubature nodes
end

%--------------------------------------------------------------------------
% MAIN PROGRAM STARTS HERE.
%--------------------------------------------------------------------------

% Define polygon and its vertices
polygon_sides=define_polygon(polygon_type);
xv=polygon_sides(:,1); yv=polygon_sides(:,2);
xmin=min(xv); xmax=max(xv); ymin=min(yv); ymax=max(yv);

% Define pointset in the polygon (cubature nodes)
P = haltonset(2);
polygon_area=polyarea(xv,yv);
rect_area=(xmax-xmin)*(ymax-ymin);
area_rat=ceil(rect_area/polygon_area);
P0=P(1:10*area_rat*cardinality_nodes,:);
P00=[xmin+(xmax-xmin)*P0(:,1) ymin+(ymax-ymin)*P0(:,2)];
in = inpolygon(P00(:,1),P00(:,2),xv,yv); Iin=find(in == 1);
P000=P00(Iin,:);
X=P000(1:cardinality_nodes,1); Y=P000(1:cardinality_nodes,2);

if abs(length(X) - cardinality_nodes) > 0
    error('Few nodes');
end

% define function
f=integrands(function_type);

% compute cubature
tic;
W=TPS_weights(xv,yv,X,Y);
wCPU=toc;
fXY=feval(f,X,Y);
cubature_value=W'*fXY;

% compute approximation of the right value
[xyw,xvc,yvc,pgon,tri]=polygauss_2018(100,xv,yv);
fxy=feval(f,xyw(:,1),xyw(:,2));
IR=(xyw(:,3))'*fxy;

% display statistics
fprintf('\n \n \t *** STATISTICS ***');

fprintf('\n \t POLYGON TYPE : %2.0f',polygon_type);
fprintf('\n \t FUNCTION TYPE: %2.0f',function_type);
fprintf('\n \t NODES CARDIN.: %4.0f',cardinality_nodes);
fprintf('\n \n \t CUBATURE RES.: %1.15e',cubature_value);
fprintf('\n \t EXACT VALUE  : %1.15e',IR);
ae=abs(IR-cubature_value);
fprintf('\n \n \t ABSOLUTE ERR.: %1.3e',ae);
if abs(IR) > 0
    fprintf('\n \t RELATIVE ERR.: %1.3e',ae/abs(IR));
end
fprintf('\n \n \t * CUBATURE CONDITIONING  : %1.15e',sum(abs(W))/sum(W));
fprintf('\n \t * CUBATURE CPUTIME  : %1.3e',wCPU);
fprintf('\n \n');

% plot domain and nodes
clf;
hold on;
axis square;
plot(xv,yv,'k-','LineWidth',4);
plot(X,Y,'r+');
hold off;

function polygon_sides=define_polygon(polygon_type)

%--------------------------------------------------------------------------
% POLYGON DEFINITION.
%--------------------------------------------------------------------------
%
% INPUT:
%
% [polygon_type]: PARAMETER THAT CHOOSES THE POLYGON.
%
% OUTPUT:
% [polygon_sides]: DEFINES THE COORDINATES OF THE VERTICES (REPEATING AT THE END
%                  THE FIRST POINT). THE VARIABLE "polygon_type" IS DEFINED ABOVE;
%                  "boundary_pts" IS A VECTOR HAVING AS COMPONENTS THE
%                  VERTICES OF THE COMPONENT, DESCRIBED COUNTERCLOCKWISE.
%                  POLYGON IS DEFINED INSIDE THE SQUARE [0,1] x [0,1].
%
%-------------------------------------------------------------------------------
% OBSERVE THAT THE FIRST VERTEX IS NOT REPEATED.
% FOR EXAMPLE, THE UNIT SQUARE IS DEFINED AS "[0 0; 1 0; 1 1; 0 1]"
% AND NOT "[0 0; 1 0; 1 1; 0 1; 0 0]".
%-------------------------------------------------------------------------------

switch polygon_type
    case 0
        fprintf('\n \t [POLYGON]: SHIFTED UNIT SQUARE [0,1]^2');
        polygon_sides=[0 0; 1 0; 1 1; 0 1]-1/2; % [-1/2 1/2; -1/2 -1/2; 1/2 -1/2; 1/2 1/2];
    case 1
        fprintf('\n \t [POLYGON]: UNIT SQUARE [0,1]^2');
        k=1;
        polygon_sides=k*[0 0; 1 0; 1 1; 0 1];
    case 2
        fprintf('\n \t [POLYGON]: CONVEX POLYGON');
        polygon_sides=[0.1 0; 0.7 0.2; 1 0.5; 0.75 0.85; 0.5 1; 0 0.25];
    case 3
        fprintf('\n \t [POLYGON]: NON CONVEX POLYGON');
        polygon_sides=(1/4)*[1 0; 3 2; 3 0; 4 2; 3 3; 3 0.85*4; 2 4; 0 3; 1 2];
    case 4
        polygon_sides=(1/4)*[1 0; 3 2; 3 0; 4 2; 3 3; 3 4; 2 4; 0 3; 1 2];
    case 5
        polygon_sides=(1/4)*[0 0; 1 2; 2 0; 3 2; 4 0; 4 4; 3 2; 2 4; 1 2; 0 4];
    case 6
        fprintf('\n \t [POLYGON]: UNIT SQUARE [-1,1]^2');
        polygon_sides=[-1 -1; 1 -1; 1 1; -1 1];
    case 7
        fprintf('\n \t [POLYGON]: UNIT TRIANGLE');
        polygon_sides=[0 0; 1 0; 1 1];
    case 8
        fprintf('\n \t [POLYGON]: PSEUDO-CIRCLE');
        N=20; a=0; b=2*pi;
        h=(b-a)/N; theta=(a:h:b-h)';
        x=cos(theta); y=sin(theta);
        x=0.5*x+0.5; y=0.5*y+0.5;
        polygon_sides=[x y];
end

%--------------------------------------------------------------------------
% "CLOSE" THE POLYGON (IT IS POSSIBLY NEEDED BY INPOLYGON).
%--------------------------------------------------------------------------
polygon_sides=[polygon_sides; polygon_sides(1,:)];





function f=integrands(function_type)

%--------------------------------------------------------------------------
% FUNCTION DEFINITION.
%--------------------------------------------------------------------------
% INPUT:
% [function_type]: PARAMETER THAT CHOOSES THE FUNCTION.
%
% OUTPUT:
%
% f: bivariate function
%
%--------------------------------------------------------------------------

switch function_type
    case 1
        f=@(x,y) franke(x,y);
    case 2
        f=@(x,y) ( (x-0.5).^2 +(y-0.5).^2 ).^(1/2);
    case 3
        f=@(x,y) (x+y).^19;
    case 4
        f=@(x,y) exp(x-0.5).^2+(y-0.5).^2;
    case 5
        f=@(x,y) exp(-100*((x-0.5).^2+(y-0.5).^2));
    case 6
        f=@(x,y) cos(30*(x+y));
    case 7
        f=@(x,y) ones(size(x))+zeros(size(y));
    case 8
        f=@(x,y) exp(5*(x-y));
    case 9
        f=@(x,y) exp(-x.^2-y.^2);
    case 10
        f=@(x,y) 1./(1+25*(x.^2+y.^2));
    case 11
        f=@(x,y) 1./(1+25*(abs(x)+abs(y)));
    case 12
        f=@(x,y) cos(20*(x+y));
end
